#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include "db.h"
#include "itemset.h"
#include "ver0.h"
/* #include "mpi.h"
 */

/* 
 * This is the first version of association mining
 * which reads data from a database input file
 * and stores the data in the defined structure array.
 *
 * The program assumes the format of the input file
 * is correct.
 */

/* variables used in MPI */
int myId, pNum;

main( int argc, char * argv[] )
{
  char fName[20];
  FILE *inFp;
  DBASE * db;
  int * dbTable;
  ITLSTNODE *minLnkLst;

  /* the table allocated for a node */
  int *localTable, localRowNum, localColNum;

  /* MPI_Status status; */

  /* setup the environment */
  MPI_Init( &argc, &argv );
  MPI_Comm_size( MPI_COMM_WORLD, &pNum );
  MPI_Comm_rank( MPI_COMM_WORLD, &myId );

  /* printf( "**************************************** \n" );
  printf( "Qiang Zheng's association mining program \n" );
  printf( "Please specify input data file name: \n" );
  scanf( "%s", fName );
  */

  if( myId==0 ) {
    strcpy( fName,"input" );

    inFp = fopen(fName, "r");

    if ( inFp == NULL ) {
      printf( "Cannot open file %s\n", fName );
      return 1;
    }

    /* read from file */
    db = readFrom( inFp ); 
    localRowNum=(db->numOfRecs)/pNum;
    localColNum=(db->numOfItems);
    dbTable=db->table;
  }

  /* distribute the database information to nodes in cluster */
  MPI_Bcast( &localRowNum, 1, MPI_INT, 0, MPI_COMM_WORLD );
  MPI_Bcast( &localColNum, 1, MPI_INT, 0, MPI_COMM_WORLD );
  localTable=( int * ) malloc( localRowNum*localColNum*sizeof( int ) );
  MPI_Scatter( dbTable, localRowNum*localColNum, MPI_INT
    , localTable, localRowNum*localColNum, MPI_INT, 0, MPI_COMM_WORLD );

  minLnkLst = newMinItSetList( localTable, localRowNum, localColNum );

  if( myId==0 ) {
    showMinItSets( minLnkLst );

    /* house cleaning */
    delDb( db );
    fclose( inFp );
  }

  free(minLnkLst);

  free( localTable );
  MPI_Finalize();
}
